@@ -97,7 +97,7 @@ typedef enum {
         GNOME_APP_CONFIGURABLE_ITEM_PRINT,
         GNOME_APP_CONFIGURABLE_ITEM_PRINT_SETUP,
         GNOME_APP_CONFIGURABLE_ITEM_CLOSE,
-        GNOME_APP_CONFIGURABLE_ITEM_EXIT,
+        GNOME_APP_CONFIGURABLE_ITEM_QUIT,
         GNOME_APP_CONFIGURABLE_ITEM_CUT,
 	/* 10 */
         GNOME_APP_CONFIGURABLE_ITEM_COPY,
@@ -126,6 +126,7 @@ typedef enum {
 	GNOME_APP_CONFIGURABLE_ITEM_END_GAME
 } GnomeUIInfoConfigurableTypes;
 
+#define GNOME_APP_CONFIGURABLE_ITEM_EXIT	GNOME_APP_CONFIGURABLE_ITEM_QUIT
 
 /* These values identify the type of pixmap used in an item */
 typedef enum {
@@ -338,11 +339,18 @@ typedef struct {
           GNOME_APP_PIXMAP_NONE, NULL,                                      \
           GNOME_APP_CONFIGURABLE_ITEM_CLOSE, (GdkModifierType) 0, NULL }
 
-#define GNOMEUIINFO_MENU_EXIT_ITEM(cb, data)                                \
+#define GNOMEUIINFO_MENU_EXIT_ITEM(cb, data)	GNOMEUIINFO_MENU_QUIT_ITEM(cb, data)
+#define GNOMEUIINFO_MENU_QUIT_ITEM(cb, data)                                \
         { GNOME_APP_UI_ITEM_CONFIGURABLE, NULL, NULL,                       \
           (gpointer)cb, (gpointer)(data), NULL,                             \
           GNOME_APP_PIXMAP_NONE, NULL,                                      \
-          GNOME_APP_CONFIGURABLE_ITEM_EXIT, (GdkModifierType) 0, NULL }
+          GNOME_APP_CONFIGURABLE_ITEM_QUIT, (GdkModifierType) 0, NULL }
+
+#define GNOMEUIINFO_MENU_QUIT_ITEM(cb, data)                                \
+        { GNOME_APP_UI_ITEM_CONFIGURABLE, NULL, NULL,                       \
+          (gpointer)cb, (gpointer)(data), NULL,                             \
+          GNOME_APP_PIXMAP_NONE, NULL,                                      \
+          GNOME_APP_CONFIGURABLE_ITEM_QUIT, (GdkModifierType) 0, NULL }
 /*
  * The "Edit" menu
  */
