@@ -52,6 +52,8 @@
 #include <libgnome/gnome-util.h>
 #include "libgnomeuiP.h"
 
+#include "libgnomeui-access.h"
+
 #include "gnome-file-entry.h"
 
 struct _GnomeFileEntryPrivate {
@@ -160,7 +162,7 @@ gnome_file_entry_mnemonic_activate (GtkWidget *widget,
 {
 	gboolean handled;
 	GnomeFileEntry *entry;
-	
+
 	entry = GNOME_FILE_ENTRY (widget);
 
 	group_cycling = group_cycling != FALSE;
@@ -179,13 +181,13 @@ gnome_file_entry_class_init (GnomeFileEntryClass *class)
 	GtkObjectClass *object_class;
 	GObjectClass *gobject_class;
 	GtkWidgetClass *widget_class;
-	
+
 	parent_class = g_type_class_peek_parent (class);
 
 	object_class = (GtkObjectClass *) class;
 	gobject_class = (GObjectClass *) class;
 	widget_class = (GtkWidgetClass *) class;
-	
+
 	object_class->destroy = gnome_file_entry_destroy;
 
 	gobject_class->finalize = gnome_file_entry_finalize;
@@ -193,7 +195,7 @@ gnome_file_entry_class_init (GnomeFileEntryClass *class)
 	gobject_class->get_property = fentry_get_property;
 
 	widget_class->mnemonic_activate = gnome_file_entry_mnemonic_activate;
-	
+
 	gnome_file_entry_signals[BROWSE_CLICKED_SIGNAL] =
 		g_signal_new("browse_clicked",
 			     G_TYPE_FROM_CLASS (gobject_class),
@@ -457,6 +459,7 @@ build_filename (GnomeFileEntry *fentry)
 {
 	const char *text;
 	char *file;
+	int len;
 
 	g_return_val_if_fail (fentry != NULL, NULL);
 	g_return_val_if_fail (GNOME_IS_FILE_ENTRY (fentry), NULL);
@@ -471,12 +474,21 @@ build_filename (GnomeFileEntry *fentry)
 	if (file == NULL)
 		return NULL;
 
-	/* Now append an '/' if it doesn't exist and we're in directory only mode */
-	if (fentry->_priv->directory_entry && strlen(file) > 0 &&
-	    file[strlen (file)] != G_DIR_SEPARATOR ) {
+	/* Now append a '/' if it doesn't exist and we're in directory only
+	 * mode.  We also have to do this if the file exists and it *is* a
+	 * directory.  Otherwise if filename is "/foo/bar/baz", the file
+	 * selector will in "/foo/bar" and put "baz" as in the filename entry
+	 * box, while "/foo/bar/baz/" is really a directory.
+	 */
+
+	len = strlen (file);
+
+	if (len != 0
+	    && file[len - 1] != G_DIR_SEPARATOR
+	    && (fentry->_priv->directory_entry || g_file_test (file, G_FILE_TEST_IS_DIR))) {
 		gchar *tmp;
 
-		tmp = g_strconcat (file, "/", NULL);
+		tmp = g_strconcat (file, G_DIR_SEPARATOR_S, NULL);
 
 		g_free (file);
 		return tmp;
@@ -521,8 +533,10 @@ browse_clicked(GnomeFileEntry *fentry)
 				 ! fentry->_priv->directory_entry);
 
 	p = build_filename (fentry);
-	gtk_file_selection_set_filename (fs, p);
-	g_free (p);
+	if (p != NULL) {
+		gtk_file_selection_set_filename (fs, p);
+		g_free (p);
+	}
 
 	g_signal_connect (fs->ok_button, "clicked",
 			  G_CALLBACK (browse_dialog_ok),
@@ -628,6 +642,8 @@ gnome_file_entry_init (GnomeFileEntry *fentry)
 	fentry->_priv->directory_entry = FALSE;
 
 	fentry->_priv->gentry = gnome_entry_new (NULL);
+	_add_atk_name_desc (fentry->_priv->gentry, _("Path"), _("Path to file"));
+
 	the_gtk_entry = gnome_file_entry_gtk_entry (fentry);
 
 	g_signal_connect (the_gtk_entry, "changed",
@@ -654,11 +670,17 @@ gnome_file_entry_init (GnomeFileEntry *fentry)
 	gtk_widget_show (fentry->_priv->gentry);
 
 	button = gtk_button_new_with_mnemonic (_("_Browse..."));
+	_add_atk_description (button, _("Pop up a file selector to choose a file"));
+
 	g_signal_connect (button, "clicked",
 			  G_CALLBACK (browse_clicked_signal),
 			  fentry);
 	gtk_box_pack_start (GTK_BOX (hbox), button, FALSE, FALSE, 0);
 	gtk_widget_show (button);
+
+	_add_atk_relation (button, the_gtk_entry,
+			   ATK_RELATION_CONTROLLER_FOR,
+			   ATK_RELATION_CONTROLLED_BY);
 }
 
 static void
