@@ -56,11 +56,13 @@
 #include <libgnome/gnome-util.h>
 #include <libgnome/gnome-config.h>
 #include <libgnomevfs/gnome-vfs-ops.h>
+#include <libgnomevfs/gnome-vfs-utils.h>
 
 #include "gnome-file-entry.h"
 #include "gnome-icon-list.h"
 #include "gnome-icon-sel.h"
 #include "gnome-icon-entry.h"
+#include "libgnomeui-access.h"
 #include <sys/stat.h>
 #include <unistd.h>
 #include <string.h>
@@ -106,8 +108,8 @@ static void ientry_set_property		(GObject *object, guint param_id,
 					 const GValue *value, GParamSpec *pspec);
 static void ientry_get_property		(GObject *object, guint param_id,
 					 GValue *value, GParamSpec *pspec);
-
 static void ientry_browse               (GnomeIconEntry *ientry);
+static gboolean ientry_mnemonic_activate (GtkWidget *widget, gboolean group_cycling);
 
 static GtkTargetEntry drop_types[] = { { "text/uri-list", 0, 0 } };
 
@@ -136,7 +138,8 @@ gnome_icon_entry_class_init (GnomeIconEntryClass *class)
 {
 	GtkObjectClass *object_class = (GtkObjectClass *)class;
 	GObjectClass *gobject_class = (GObjectClass *)class;
-
+	GtkWidgetClass *widget_class = (GtkWidgetClass *)class;
+	
 	gnome_ientry_signals[CHANGED_SIGNAL] =
 		g_signal_new("changed",
 			     G_TYPE_FROM_CLASS (gobject_class),
@@ -163,6 +166,8 @@ gnome_icon_entry_class_init (GnomeIconEntryClass *class)
 	gobject_class->set_property = ientry_set_property;
 	gobject_class->get_property = ientry_get_property;
 
+	widget_class->mnemonic_activate = ientry_mnemonic_activate;
+	
 	g_object_class_install_property (gobject_class,
 					 PROP_HISTORY_ID,
 					 g_param_spec_string (
@@ -389,6 +394,9 @@ entry_activated(GtkWidget *widget, GnomeIconEntry *ientry)
 		gnome_icon_selection_show_icons (GNOME_ICON_SELECTION (priv->icon_sel));
 	} else {
 		/* We pretend like ok has been called */
+		g_free (priv->picked_file);
+		priv->picked_file = g_strdup (filename);
+
 		update_icon (ientry);
 		gtk_widget_hide (ientry->_priv->pick_dialog);
 	}
@@ -532,8 +540,6 @@ browse_clicked(GnomeFileEntry *fentry, GnomeIconEntry *ientry)
 	g_object_set_data(G_OBJECT(w),"fs",fs);
 
 	g_object_set_data(G_OBJECT(fs->file_list),"frame",w);
-	g_signal_connect (fs->file_list, "select_row",
-			  G_CALLBACK (setup_preview), NULL);
 	g_object_set_data(G_OBJECT(fs->selection_entry),"frame",w);
 
 	closure = g_cclosure_new (G_CALLBACK (setup_preview), NULL, NULL);
@@ -643,6 +649,25 @@ delete_event_handler (GtkWidget   *widget,
 	return TRUE;
 }
 
+static gboolean
+ientry_mnemonic_activate (GtkWidget *widget,
+			  gboolean   group_cycling)
+{
+	gboolean handled;
+	GnomeIconEntry *entry;
+	
+	entry = GNOME_ICON_ENTRY (widget);
+
+	group_cycling = group_cycling != FALSE;
+
+	if (!GTK_WIDGET_IS_SENSITIVE (entry->_priv->pickbutton))
+		handled = TRUE;
+	else
+		g_signal_emit_by_name (entry->_priv->pickbutton, "mnemonic_activate", group_cycling, &handled);
+
+	return handled;
+}
+
 static void
 ientry_browse(GnomeIconEntry *ientry)
 {
@@ -722,7 +747,17 @@ ientry_browse(GnomeIconEntry *ientry)
 				  G_CALLBACK (gtk_widget_destroyed),
 				  &priv->pick_dialog);
 
+		/* Set up accessible properties for the pick dialog */
+		_add_atk_name_desc (priv->pick_dialog,
+				    _("Icon selection dialog"),
+				    _("This dialog box lets you select an icon."));
+		_add_atk_relation (priv->pickbutton, priv->pick_dialog,
+				   ATK_RELATION_CONTROLLER_FOR, ATK_RELATION_CONTROLLED_BY);
+
 		priv->icon_sel = gnome_icon_selection_new ();
+		_add_atk_name_desc (priv->icon_sel,
+				    _("Icon selector"),
+				    _("Please pick the icon you want."));
 
 		gnome_icon_selection_add_directory (GNOME_ICON_SELECTION (priv->icon_sel),
 						    priv->pick_dialog_dir);
@@ -780,26 +815,33 @@ drag_data_received (GtkWidget        *widget,
 		    guint32           time,
 		    GnomeIconEntry   *ientry)
 {
-	GList *uris, *li;
-	GnomeVFSURI *uri = NULL;
+	char **uris;
+	char *file = NULL;
 	GnomeVFSFileInfo *file_info;
+	int i;
 
-	/*here we extract the filenames from the URI-list we recieved*/
-	uris = gnome_vfs_uri_list_parse (selection_data->data);
+	uris = g_strsplit (selection_data->data, "\r\n", -1);
+	if (uris == NULL) {
+		gtk_drag_finish (context, FALSE, FALSE, time);
+		return;
+	}
 
-	for (li = uris; li != NULL; li = li->next) {
+	for (i =0; uris[i] != NULL; i++) {
 		const char *mimetype;
-		const char *path;
-
-		uri = li->data;
-
+		GnomeVFSURI *uri;
 		/* FIXME: Support non-local files */
-		if ( ! gnome_vfs_uri_is_local (uri)) {
-			uri = NULL;
+		/* FIXME: Multiple files */
+		file = gnome_vfs_get_local_path_from_uri (uris[i]);
+		if (file == NULL)
+			continue;
+
+		uri = gnome_vfs_uri_new (uris[i]);
+		if (uri == NULL) {
+			g_free (file);
+			file = NULL;
 			continue;
 		}
 
-		path = gnome_vfs_uri_get_path (uri);
 		file_info = gnome_vfs_file_info_new ();
 		/* FIXME: OK to do synchronous I/O on a possibly-remote URI here? */
 		gnome_vfs_get_file_info_uri (uri, file_info,
@@ -814,14 +856,14 @@ drag_data_received (GtkWidget        *widget,
 
 			/* Try to read the .desktop's Icon entry */
 			confpath = g_strdup_printf ("=%s=/Desktop Entry/Icon",
-						    path);
+						    file);
 			icon = gnome_config_get_string (confpath);
 			if (icon == NULL || *icon == '\0') {
 				gnome_config_drop_file (confpath);
 				g_free (confpath);
 				g_free (icon);
 				confpath = g_strdup_printf ("=%s=/KDE Desktop Entry/Icon",
-							    path);
+							    file);
 				icon = gnome_config_get_string (confpath);
 				gnome_config_drop_file (confpath);
 				g_free (confpath);
@@ -832,7 +874,8 @@ drag_data_received (GtkWidget        *widget,
 			}
 
 			if (icon == NULL) {
-				uri = NULL;
+				g_free (file);
+				file = NULL;
 				continue;
 			}
 
@@ -866,25 +909,22 @@ drag_data_received (GtkWidget        *widget,
 				}
 				g_free (full);
 			}
-		} else if (gnome_icon_entry_set_filename (ientry, path)) {
+		} else if (gnome_icon_entry_set_filename (ientry, file)) {
 			gnome_vfs_file_info_unref (file_info);
 			break;
 		}
-		uri = NULL;
+		g_free (file);
+		file = NULL;
 		gnome_vfs_file_info_unref (file_info);
 	}
 
+	g_strfreev (uris);
 
 	/*if there's isn't a file*/
-	if (uri == NULL) {
+	if (file == NULL) {
 		gtk_drag_finish (context, FALSE, FALSE, time);
-		/*free the list of files we got*/
-		gnome_vfs_uri_list_free (uris);
 		return;
 	}
-
-	/*free the list of files we got*/
-	gnome_vfs_uri_list_free (uris);
 }
 
 static void
@@ -939,6 +979,11 @@ gnome_icon_entry_instance_init (GnomeIconEntry *ientry)
 	gtk_widget_show(w);
 	gtk_box_pack_start (GTK_BOX (ientry), w, TRUE, TRUE, 0);
 	ientry->_priv->pickbutton = gtk_button_new_with_label(_("No Icon"));
+	/* Set our accessible name and description */
+	_add_atk_name_desc (GTK_WIDGET (ientry->_priv->pickbutton),
+			    _("Icon Selector"),
+			    _("This button will open a window to let you select an icon."));
+
 	gtk_drag_dest_set (GTK_WIDGET (ientry->_priv->pickbutton),
 			   GTK_DEST_DEFAULT_MOTION |
 			   GTK_DEST_DEFAULT_HIGHLIGHT |
@@ -961,6 +1006,10 @@ gnome_icon_entry_instance_init (GnomeIconEntry *ientry)
 	gtk_widget_show (ientry->_priv->pickbutton);
 
 	ientry->_priv->fentry = gnome_file_entry_new (NULL, _("Browse"));
+	_add_atk_name_desc (ientry->_priv->fentry,
+			    _("Icon path"),
+			    _("Here you should enter the name of the directory "
+			      "where icon images are located."));
 	/*BORPORP */
 	gnome_file_entry_set_modal (GNOME_FILE_ENTRY (ientry->_priv->fentry),
 				    TRUE);
