@@ -39,6 +39,7 @@
 #include "gnome-uidefs.h"
 
 #include "gnome-icon-sel.h"
+#include "libgnomeui-access.h"
 
 #include <libgnomevfs/gnome-vfs-ops.h>
 
@@ -311,9 +312,6 @@ gnome_icon_selection_add_directory (GnomeIconSelection * gis,
     GnomeVFSFileInfo *info;
     gchar *full_path;
     
-#ifdef GNOME_ENABLE_DEBUG
-    g_print("File: %s\n", de->d_name);
-#endif
     if ( *(de->d_name) == '.' ) continue; /* skip dotfiles */
 
     full_path = g_build_filename (dir, de->d_name, NULL);
@@ -329,9 +327,7 @@ gnome_icon_selection_add_directory (GnomeIconSelection * gis,
     if (mimetype != NULL &&
 	strncmp(mimetype, "image", strlen("image")) == 0 ) {
       gchar * full_path = g_build_filename (dir, de->d_name, NULL);
-#ifdef GNOME_ENABLE_DEBUG
-      g_print("Full path: %s\n", full_path);
-#endif
+
       if (g_file_test (full_path, G_FILE_TEST_IS_REGULAR)) {
 	      /* Image filename, exists, regular file, go for it. */
 	      gis->_priv->file_list =
@@ -409,6 +405,8 @@ gnome_icon_selection_show_icons (GnomeIconSelection * gis)
 		return;
 
 	label = gtk_label_new (_("Loading Icons..."));
+	_add_atk_relation (GTK_WIDGET (gis), label,
+			   ATK_RELATION_LABELLED_BY, ATK_RELATION_LABEL_FOR);
 	gtk_box_pack_start (GTK_BOX (gis->_priv->box),
 			    label, FALSE, FALSE, 0);
 	gtk_widget_show (label);
@@ -417,6 +415,8 @@ gnome_icon_selection_show_icons (GnomeIconSelection * gis)
 			  &label);
 
 	progressbar = gtk_progress_bar_new ();
+	_add_atk_relation (progressbar, label,
+			   ATK_RELATION_LABELLED_BY, ATK_RELATION_LABEL_FOR);
 	gtk_box_pack_start (GTK_BOX (gis->_priv->box),
 			    progressbar, FALSE, FALSE, 0);
 	gtk_widget_show (progressbar);
